/*
  A5020.cpp - Mouse sensor library
  Copyright (C) 2009 Aleksi Pihkanen.  All rights reserved.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  See file LICENSE.txt for further informations on licensing terms.
  
  This library modified by The-MenZ(http://www.the-menz.com/).
  
  Changes:
  30th Oct, 2013:Change the include file "WConstants.h" to "Arduino.h"
                 for compiling on Arduino SDK 1.0 or later.
                 If you use previous Arduino SDK, please use original library.
*/

#include "Arduino.h"
//#include "WConstants.h" // If you're not using Arduino, substitute this for the above.
#include "A5020.h"

A5020::A5020(uint8_t c,uint8_t d,uint8_t e){
  _clock = c; _data = d; _enable = e;
  delta_x = 0;
  delta_y = 0;
  squal = 0;
  pinMode(_clock, OUTPUT);
  pinMode(_data, OUTPUT);
  pinMode(_enable, OUTPUT);
  digitalWrite(_enable, HIGH);
}

void A5020::pushbyte(uint8_t c){
  pinMode(_data, OUTPUT);
  for(i=0x80;i;i=i>>1){
    digitalWrite(_clock, LOW);
    digitalWrite(_data, c & i);
    digitalWrite(_clock, HIGH);
  }
}

int8_t A5020::pullbyte(){
  int8_t ret=0;
  pinMode(_data, INPUT);
  for(i=0x80;i;i=i>>1){
    digitalWrite(_clock, LOW);
    ret |= i*digitalRead(_data);
    digitalWrite(_clock, HIGH);
  }
  return(ret);
}

void A5020::read(){
  digitalWrite(_enable, LOW);
  pushbyte(0x02);
  motion = pullbyte() & 0x80;
  digitalWrite(_enable, HIGH);
  if(motion){
    digitalWrite(_enable, LOW);
    pushbyte(0x63);
    delta_x = pullbyte();
    delta_y = pullbyte();
    squal = (uint8_t) pullbyte();
    digitalWrite(_enable, HIGH);
  }
  else delta_x = delta_y = 0;
}

void A5020::reset(){
  // Initiate chip reset
  digitalWrite(_enable, LOW);
  pushbyte(0xfa);
  pushbyte(0x5a);
  digitalWrite(_enable, HIGH);
  // Set 1000cpi resolution
  digitalWrite(_enable, LOW);
  pushbyte(0x0d);
  pushbyte(0x01);
  digitalWrite(_enable, HIGH);
}
